
#include <cstdlib>
#include <cassert>
#include "avl_tree.hpp"


using boost::shared_ptr;


template<typename Key>
avl_tree<Key>::avl_tree() : _root()
{
}


template<typename Key>
avl_tree<Key>::~avl_tree()
{
	cout << "avl_tree::~avl_tree():" << endl;
	destroy_subtree(_root);
}


template<typename Key>
shared_ptr<node<Key> > avl_tree<Key>::find(Key key)
{
	if (_root.use_count() == 0)
		return shared_ptr<node<Key> >();
	
	shared_ptr<node<Key> > current(_root);
	while (current.use_count() != 0)
		if (key < current->key)
			current = current->left;
		else if (key > current->key)
			current = current->right;
		else
			return current;

	return shared_ptr<node<Key> >();
}


template<typename Key>
shared_ptr<node<Key> > avl_tree<Key>::predecessor(shared_ptr<node<Key> > a_node)
{
	if (a_node.use_count() == 0 || _root.use_count() == 0|| a_node->left.use_count() == 0)
		return shared_ptr<node<Key> >();
	
	shared_ptr<node<Key> > pred = a_node->left;
	while (pred->right.get() != 0)
		pred = pred->right;
	return pred;
}


template<typename Key>
shared_ptr<node<Key> > avl_tree<Key>::successor(shared_ptr<node<Key> > a_node)
{
	if (a_node.use_count() == 0 || _root.use_count() == 0 || a_node->right.use_count() == 0)
		return shared_ptr<node<Key> >();
	
	shared_ptr<node<Key> > succ = a_node->right;
	while (succ->left.get() != 0)
		succ = succ->left;
	return succ;
}


template<typename Key>
void avl_tree<Key>::insert(Key key)
{
	shared_ptr<node<Key> > new_node(new node<Key>(key));
	
	// empty tree is trivial case
	if (_root.get() == 0)
	{
		_root = new_node;
		return;
	}
	
	shared_ptr<node<Key> > current(_root);
	shared_ptr<node<Key> > last_ancestor; // last ancestor with non-zero balance
	// insert new_node into an empty place
	while (true)
	{
		if (current->balance != 0)
			last_ancestor = current;
		if (key < current->key)
		{
			if (current->left.get() == 0)
			{
				current->left = new_node;
				new_node->parent = current;
				break;
			}
			current = current->left;
		}
		else
		{
			if (current->right.get() == 0)
			{
				current->right = new_node;
				new_node->parent = current;
				break;
			}
			current = current->right;
		}
	}
	
	// just modify balances
	if (last_ancestor.get() == 0 || last_ancestor->balance == 0)
	{
		current = new_node;
		do
		{
			if (current == current->parent->left)
				--current->parent->balance;
			else if (current == current->parent->right)
				++current->parent->balance;
			else
				assert(false);
			current = current->parent;
		}
		while (current != _root);
		return;
	}
	
	// modify balances from new_node to last_ancestor
	current = new_node;
	do
	{
		if (current == current->parent->left)
			--current->parent->balance;
		else if (current == current->parent->right)
			++current->parent->balance;
		else
			assert(false);
		current = current->parent;
	}
	while (current != last_ancestor);
	
	// fix balance factors
	current = new_node;
	if (is_left_descendant(last_ancestor, current))
	{
		// node inserted to the left
		shared_ptr<node<Key> > ancestor_l = last_ancestor->left;
		shared_ptr<node<Key> > ancestor_lr = ancestor_l->right;
		if (last_ancestor->balance == -2 && ancestor_l->balance == -1)
			case_l1(last_ancestor);
		else if (last_ancestor->balance == -2 && ancestor_l->balance == +1 && ancestor_lr->balance == -1)
			case_l2(last_ancestor);
		else if (last_ancestor->balance == -2 && ancestor_l->balance == +1 && ancestor_lr->balance == +1)
			case_l3(last_ancestor);
		else if (last_ancestor->balance == -2 && ancestor_l->balance == +1 && ancestor_lr->balance == 0)
			case_l4(last_ancestor);
	}
	else
	{
		// node inserted to the right
		shared_ptr<node<Key> > ancestor_r = last_ancestor->right;
		shared_ptr<node<Key> > ancestor_rl = ancestor_r->left;
		if (last_ancestor->balance == +2 && ancestor_r->balance == +1)
			case_r1(last_ancestor);
		else if (last_ancestor->balance == +2 && ancestor_r->balance == -1 && ancestor_rl->balance == +1)
			case_r2(last_ancestor);
		else if (last_ancestor->balance == +2 && ancestor_r->balance == -1 && ancestor_rl->balance == -1)
			case_r3(last_ancestor);
		else if (last_ancestor->balance == +2 && ancestor_r->balance == -1 && ancestor_rl->balance == 0)
			case_r4(last_ancestor);
	}
}


template<typename Key>
void avl_tree<Key>::remove(Key key)
{
	if (_root.use_count() == 0)
		return;
	
	shared_ptr<node<Key> > del_node = find(key);
	if (del_node.use_count() == 0)
		return;
	
	if (del_node == _root && _root.use_count() == 1)
	{
		_root.reset();
		return;
	}

	shared_ptr<node<Key> > succ(successor(del_node));
	shared_ptr<node<Key> > pred(predecessor(del_node));
	shared_ptr<node<Key> > succ_pred_p; // parent of successor or predecessor
	if (succ.get() == 0 && pred.get() == 0)
	{
		succ_pred_p = del_node->parent;
		if (del_node == succ_pred_p->left)
		{
			++succ_pred_p->balance;
			succ_pred_p->left.reset();
		}
		else if (del_node == succ_pred_p->right)
		{
			--succ_pred_p->balance;
			succ_pred_p->right.reset();
		}
		else
			assert(false);
	}
	else if (succ.get() != 0)
	{
		del_node->key = succ->key;
		succ_pred_p = succ->parent;// could be also succ_pred_p == del_node->parent
		if (succ == succ_pred_p->left) // successor is not sibling of del_node
		{
			// connect succ_pred_p with single succ child (if exists)
			if (succ->right.get() != 0)
			{
				succ_pred_p->left = succ->right;
				succ->right->parent = succ_pred_p;
			}
			else
				succ_pred_p->left.reset();
			++succ_pred_p->balance;
		}
		else if (succ == succ_pred_p->right) // successor is sibling of del_node
		{
			// connect succ_pred_p with single succ child (if exists)
			if (succ->right.get() != 0)
			{
				succ_pred_p->right = succ->right;
				succ->right->parent = succ_pred_p;
			}
			else
				succ_pred_p->right.reset();
			--succ_pred_p->balance;
		}
	}
	else if (pred.get() != 0)
	{
		del_node->key = pred->key;
		succ_pred_p = pred->parent;// could be also succ_pred_p == del_node->parent
		if (pred == succ_pred_p->right) // successor is not sibling of del_node
		{
			// connect succ_pred_p with single pred child (if exists)
			if (pred->left.get() != 0)
			{
				succ_pred_p->right = pred->left;
				pred->left->parent = succ_pred_p;
			}
			else
				succ_pred_p->right.reset();
			--succ_pred_p->balance;
		}
		else if (pred == succ_pred_p->left) // successor is sibling of del_node
		{
			// connect succ_pred_p with single pred child (if exists)
			if (pred->left.get() != 0)
			{
				succ_pred_p->left = pred->left;
				pred->left->parent = succ_pred_p;
			}
			else
				succ_pred_p->left.reset();
			++succ_pred_p->balance;
		}
	}

	while (succ_pred_p.get() != 0)
	{
		if (succ_pred_p->balance == -1 || succ_pred_p->balance == +1) // case D1
			break;
		else if (succ_pred_p->balance == 0) // case D2
		{
			if (succ_pred_p->parent.get() != 0)
			{
				if (succ_pred_p == succ_pred_p->parent->left)
					++succ_pred_p->parent->balance;
				else if (succ_pred_p == succ_pred_p->parent->right)
					--succ_pred_p->parent->balance;
				else
					assert(false);
			}
		}	
		else if (succ_pred_p->balance == +2) // cases R1-R5
		{
			shared_ptr<node<Key> > succ_pred_pr = succ_pred_p->right;
			if (succ_pred_pr->balance == +1)
				case_r1(succ_pred_p);
			else if (succ_pred_pr->balance == -1)
			{
				shared_ptr<node<Key> > succ_pred_prl = succ_pred_pr->left;
				if (succ_pred_prl->balance == +1)
					case_r2(succ_pred_p);
				else if (succ_pred_prl->balance == -1)
					case_r3(succ_pred_p);
				else if (succ_pred_prl->balance == 0)
					case_r4(succ_pred_p);
			}
			else if (succ_pred_pr->balance == 0)
				case_r5(succ_pred_p);
		}
		else if (succ_pred_p->balance == -2) // cases L1-L5
		{
			shared_ptr<node<Key> > succ_pred_pl = succ_pred_p->left;
			if (succ_pred_pl->balance == -1)
				case_l1(succ_pred_p);
			else if (succ_pred_pl->balance == +1)
			{
				shared_ptr<node<Key> > succ_pred_plr = succ_pred_pl->right;
				if (succ_pred_plr->balance == -1)
					case_l2(succ_pred_p);
				else if (succ_pred_plr->balance == +1)
					case_l3(succ_pred_p);
				else if (succ_pred_plr->balance == 0)
					case_l4(succ_pred_p);
			}
			else if (succ_pred_pl->balance == 0)
				case_l5(succ_pred_p);
		}
		
		succ_pred_p = succ_pred_p->parent;
	}
}


template<typename Key>
void avl_tree<Key>::rotate_left(shared_ptr<node<Key> > a_node)
{
	assert(a_node->right != 0);
	
	shared_ptr<node<Key> > node_p = a_node->parent;
	shared_ptr<node<Key> > node_r = a_node->right;
	shared_ptr<node<Key> > node_rl = a_node->right->left;
	
	node_r->parent = node_p;
	a_node->parent = node_r;
	if (node_rl.get() != 0)
		node_rl->parent = a_node;
	a_node->right = node_rl;
	node_r->left = a_node;
	if (node_p.get() != 0)
	{
		if (a_node == node_p->left)
			node_p->left = node_r;
		else if (a_node == node_p->right)
			node_p->right = node_r;
		else
			assert(false);
	}
	
	if (_root == a_node)
		_root = a_node->parent;
}


template<typename Key>
void avl_tree<Key>::rotate_right(shared_ptr<node<Key> > a_node)
{
	assert(a_node->left != 0);
	
	shared_ptr<node<Key> > node_p = a_node->parent;
	shared_ptr<node<Key> > node_l = a_node->left;
	shared_ptr<node<Key> > node_lr = a_node->left->right;
	
	node_l->parent = node_p;
	a_node->parent = node_l;
	if (node_lr.get() != 0)
		node_lr->parent = a_node;
	a_node->left = node_lr;
	node_l->right = a_node;
	if (node_p.get() != 0)
	{
		if (a_node == node_p->left)
			node_p->left = node_l;
		else if (a_node == node_p->right)
			node_p->right = node_l;
		else
			assert(false);
	}
	
	if (_root == a_node)
		_root = a_node->parent;
}


template<typename Key>
void avl_tree<Key>::case_l1(shared_ptr<node<Key> > a_node)
{
	shared_ptr<node<Key> > node_l = a_node->left;
	assert(a_node != 0 && node_l != 0);
	assert(a_node->balance == -2 && node_l->balance == -1);
	
	rotate_right(a_node);
	a_node->balance = 0;
	node_l->balance = 0;
}


template<typename Key>
void avl_tree<Key>::case_l2(shared_ptr<node<Key> > a_node)
{
	shared_ptr<node<Key> > node_l = a_node->left;
	shared_ptr<node<Key> > node_lr = node_l->right;
	assert(a_node != 0 && node_l != 0 && node_lr != 0);
	
	rotate_left(node_l);
	rotate_right(a_node);
	a_node->balance = +1;
	node_l->balance = 0;
	node_lr->balance = 0;
}


template<typename Key>
void avl_tree<Key>::case_l3(shared_ptr<node<Key> > a_node)
{
	shared_ptr<node<Key> > node_l = a_node->left;
	shared_ptr<node<Key> > node_lr = node_l->right;
	assert(a_node != 0 && node_l != 0 && node_lr != 0);
	
	rotate_left(node_l);
	rotate_right(a_node);
	a_node->balance = 0;
	node_l->balance = -1;
	node_lr->balance = 0;
}


template<typename Key>
void avl_tree<Key>::case_l4(shared_ptr<node<Key> > a_node)
{
	shared_ptr<node<Key> > node_l = a_node->left;
	shared_ptr<node<Key> > node_lr = node_l->right;
	assert(a_node != 0 && node_l != 0 && node_lr != 0);
	
	rotate_left(node_l);
	rotate_right(a_node);
	a_node->balance = 0;
	node_l->balance = 0;
	node_lr->balance = 0;
}


template<typename Key>
void avl_tree<Key>::case_l5(shared_ptr<node<Key> > a_node)
{
	shared_ptr<node<Key> > node_l = a_node->left;
	assert(a_node != 0 && node_l != 0);
	assert(a_node->balance == -2 && node_l->balance == 0);
	
	rotate_right(a_node);
	a_node->balance = -1;
	node_l->balance = +1;
}


template<typename Key>
void avl_tree<Key>::case_r1(shared_ptr<node<Key> > a_node)
{
	shared_ptr<node<Key> > node_r = a_node->right;
	assert(a_node != 0 && node_r != 0);
	assert(a_node->balance == +2 && node_r->balance == +1);
	
	rotate_left(a_node);
	a_node->balance = 0;
	node_r->balance = 0;
}


template<typename Key>
void avl_tree<Key>::case_r2(shared_ptr<node<Key> > a_node)
{
	shared_ptr<node<Key> > node_r = a_node->right;
	shared_ptr<node<Key> > node_rl = node_r->left;
	assert(a_node != 0 && node_r != 0 && node_rl != 0);
	
	rotate_right(node_r);
	rotate_left(a_node);
	a_node->balance = -1;
	node_r->balance = 0;
	node_rl->balance = 0;
}


template<typename Key>
void avl_tree<Key>::case_r3(shared_ptr<node<Key> > a_node)
{
	shared_ptr<node<Key> > node_r = a_node->right;
	shared_ptr<node<Key> > node_rl = node_r->left;
	assert(a_node != 0 && node_r != 0 && node_rl != 0);
	
	rotate_right(node_r);
	rotate_left(a_node);
	a_node->balance = 0;
	node_r->balance = +1;
	node_rl->balance = 0;
}


template<typename Key>
void avl_tree<Key>::case_r4(shared_ptr<node<Key> > a_node)
{
	shared_ptr<node<Key> > node_r = a_node->right;
	shared_ptr<node<Key> > node_rl = node_r->left;
	assert(a_node != 0 && node_r != 0 && node_rl != 0);
	
	rotate_right(node_r);
	rotate_left(a_node);
	a_node->balance = 0;
	node_r->balance = 0;
	node_rl->balance = 0;
}


template<typename Key>
void avl_tree<Key>::case_r5(shared_ptr<node<Key> > a_node)
{
	shared_ptr<node<Key> > node_r = a_node->right;
	assert(a_node != 0 && node_r != 0);
	assert(a_node->balance == +2 && node_r->balance == 0);
	
	rotate_left(a_node);
	a_node->balance = +1;
	node_r->balance = -1;
}


template<typename Key>
bool avl_tree<Key>::is_left_descendant(shared_ptr<node<Key> > ancestor, shared_ptr<node<Key> > descendant)
{
	assert(ancestor != 0 && descendant != 0 && ancestor != descendant);
	while (descendant->parent != ancestor)
		descendant = descendant->parent;
	if (descendant == ancestor->left)
		return true;
	return false;
}


template<typename Key>
void avl_tree<Key>::destroy_subtree(shared_ptr<node<Key> > root)
{
	if (root->left.get() != 0)
	{
		root->left->parent.reset();
		destroy_subtree(root->left);
		root->left.reset();
	}

	if (root->right.get() != 0)
	{
		root->right->parent.reset();
		destroy_subtree(root->right);
		root->right.reset();
	}

	root.reset();
}


int main()
{
	{
	avl_tree<int> tree;
	
	tree.insert(12);
	tree.insert(6);
	tree.insert(31);
	tree.insert(1);
	tree.insert(10);
	tree.insert(25);
	tree.insert(7);
	tree.insert(21);
	tree.insert(2);
	tree.insert(8);
	tree.insert(35);
	tree.insert(4);
	tree.insert(3);
	tree.insert(19);
	tree.insert(32);
	tree.insert(30);

	tree.remove(19);
	}

	return EXIT_SUCCESS;
}
 
